function [CR,mdat] = CRboundNQNoLinearizeWithPileup(spectran,Tintegrate,deadtime)
% function [CR,mdat] = CRboundNQNoLinearizeWithPileup(spectran,Tintegrate,deadtime)
%  Computes Cramer-Rao lower bound (CRLB) for log NQ data with pileup 
%  directly from general formula for Fisher Info matrix
%  for general Gaussian case (Eq. 3.31 of Kay). See Onenote with keyword CRLB for formula
% Also assumes counting detectors with pileup that are not Poisson
%  inputs: 
%    spectran: a spectrum struct transmitted through object
%     Tintegrate: (seconds) the integrate time for detector. 
%     deadtime: the deadtime in units compatible with count rate from specdat
%  outputs:
%    CR: the CRLB with pileup
%    mdat: debugging info struct. See code for fields
%  REA 10/1/10 - 2011-Aug-23 10:35 - 2012-May-21 10:57
% COPYRIGHT:   Aprend Technology, 2011. All rights reserved.   


    % compute eff att coeff matrix and covariance with NQ and pileup
[M,nqstats] = M_NQ_with_deadtime(spectran,Tintegrate, deadtime);
m_da_offset = 0.001*[1 1]; % offset to step off for M with deadtime calculation
dAs = [m_da_offset(1) 0; 0 m_da_offset(2)];  % offset vectors arranged in a matrix
dRdA = dRdALOC(spectran, Tintegrate, deadtime,dAs);

F = zeros(2); % second term of Fisher information matrix

RL = nqstats.RL;
RLinv = inv(RL);
   % first accumulate the derivative of covariance terms
for i = 1:2
   for j= 1:2
     F(i,j) = 0.5*trace(RLinv*dRdA(:,:,i)*RLinv*dRdA(:,:,j));
   end
end
F1 = M'*RLinv*M;
CR = inv(F1 + F);
if nargout >1 % save workspace
    CR_const_cov = inv(F1);
    MLE = CR_const_cov*M'*RLinv;
    mdat = GetWorkspace;
end

%------------------------------ LOCAL FUNCTIONS ----------------------------------
function dRdA = dRdALOC(specdat, Tintegrate, deadtime,dAs)
nbins = 2;
dRdA = zeros(nbins,nbins,2); % 2 planes, one for each A-vector component
cv0 = covLOC(specdat, Tintegrate, deadtime);
for kdim = 1:2 
    sp2 = specdat;
    sp2.specnum = exp(-specdat.mus*dAs(kdim,:)').*specdat.specnum; % note transpose
    cv = covLOC(sp2, Tintegrate, deadtime);
    dRdA(:,:,kdim) = (cv - cv0)/dAs(kdim,kdim);
end

     %------------
function RL = covLOC(specdat, Tintegrate, deadtime)
    % extract rate from integral of spectrum and integrate time
N = trapz(specdat.egys,specdat.specnum);
    % normalize the input spectrum
specdat.specnum_norm = specdat.specnum/N;
rate = N/Tintegrate;
eta = rate*deadtime;
eta1 = 1 + eta;
Ebar = trapz(specdat.egys,specdat.egys(:).*specdat.specnum_norm(:));
E2bar = trapz(specdat.egys,specdat.egys(:).^2.*specdat.specnum_norm(:));
F = E2bar/(Ebar)^2;
RL = (1/N)*[1/eta1,1/eta1;1/eta1, F] ; % covariance of log with deadtime
