function [M,mdat] = mubarNbinNdim(specdat,As,varargin)
% function [M,mdat] = mubarNbinNdim(specdat,As,varargin)
% computes effective mu matrix for specified As thicknesses. 
% this version works with an arbitrary number of basis functions
%  inputs:
%    specdat: spectrum struct with fields
%       specnum: number spectrum
%       egys: the x-ray energies
%       mus: the attenuation coefficients (negys,ndim)
%       idx_threshold: the indices for interbin energies
%    As: a matrix size(npoints,ndim)
%    idxs4bins: optional, default is to use indexes computed from idx_threshold in specdat
%    force_quiet: (optional = false) if present does not issue warnings
%  outputs:
%    M: an nspect,ndim,npoints matrix of effective mus in bins specified by specdat.idx_threshold
%    mdat: debugging info struct. See code for fields (not implemented)
%  REA 4/20-2011-Aug-1 12:02
% COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

idxs4bins = [];
force_quiet = false;
nreqargs = 2;
assert(nargin>=nreqargs);

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'force_quiet';          force_quiet = true;
     case 'idxs4bins';         idxs4bins=varargin{i+1}; assert(iscell(idxs4bins));    i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

ndim = size(specdat.mus,2);
assert(size(As,2) == ndim);

if isfield(specdat,'Ethreshold')
   if (~force_quiet) && isfield(specdat,'idx_threshold')
      warning('mubarNbinNdim: specdat contains both Ethreshold and idx_threshold--Ethreshold overrides');
   end
       % compute the indexes into the egy array for the N bins
   nbins = numel(specdat.Ethreshold) + 1;
   idx_threshold = zeros(nbins-1,1);
   for k = 1:(nbins-1)
      [minval,idx] = min(abs(specdat.Ethreshold(k) - specdat.egys));
      idx_threshold(k) = idx;
   end
   specdat.idx_threshold = idx_threshold;
end

idx_threshold = specdat.idx_threshold;       
nspect = numel(idx_threshold)+1;
       
if isempty(idxs4bins)
    bin_edges = [0; idx_threshold(:);numel(specdat.egys)];
    idxs = cell(nspect,1);
    for k = 1:nspect
        idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
    end
else % idxs were entered as a parameter
   idxs = idxs4bins;
end
npoints = size(As,1);

M = zeros(nspect,ndim,npoints);
CRLB_full_egy_info = zeros(ndim,ndim,npoints); % the CRLB with full energy information. Assume spectrum sampled at high egy resolution
lambdas = zeros(npoints,nspect); % the expected value of counts in each bin for each thickness
for kspect = 1:nspect
    idx = idxs{kspect};
    sp = specdat.specnum(idx); % un-normalized spectrum for a bin
    mus = specdat.mus(idx,:); % mus for energies in the bin
    for kt = 1:npoints
        As_pnt = As(kt,:);
        Transmission_E = exp(-mus*As_pnt(:)); % transmission vs. egy for this thickness
        spec_transmitted = sp(:).*Transmission_E(:);
        sp_norm = spec_transmitted./ sum(spec_transmitted);
        Ms = sum(mus.*repmat(sp_norm(:),1,ndim)); % size = 1,ndim
        M(kspect,:,kt) = Ms;
           % put the calculation of the CRLB with full energy info here since have transmitted spectrum
           % but only need to do it once so do  when kspect == 1
        if kspect == 1
            Cfull = diag(specdat.specnum(:).*exp(-specdat.mus*As_pnt(:))); % covariance of PHA of full spectrum. assume counting detector with NO logs
                      % see discussion in my post 29 "SNR with PHS vs. number of bins" in blog that gives same result
                      % as covariance of A vector with log data
            Mfull = specdat.mus; % assume basis functions are the columns of mus
            CRLB_full_egy_info(:,:,kt) = inv(Mfull'*Cfull*Mfull);
        end
    end
end

if nargout>1
    mdat.idxs = idxs;
    mdat.CRLB_full_egy_info = CRLB_full_egy_info;
end